package jp.deadend.noname.treenote;

import android.text.Layout;
import android.text.method.ArrowKeyMovementMethod;
import android.text.method.MovementMethod;
import android.text.Selection;
import android.text.Spannable;
import android.text.style.ClickableSpan;
import android.view.MotionEvent;
import android.widget.TextView;

public class ArrowKeyAndLinkMovementMethod extends ArrowKeyMovementMethod {
	private static ArrowKeyAndLinkMovementMethod mInstance;

	public static MovementMethod getInstance() {
		if (mInstance == null) {
			mInstance = new ArrowKeyAndLinkMovementMethod();
		}
		return mInstance;
	}

	@Override
	public boolean onTouchEvent(TextView widget, Spannable text, MotionEvent event) {
		int action = event.getAction();

		if (action == MotionEvent.ACTION_UP || action == MotionEvent.ACTION_DOWN) {
			int x = (int)event.getX();
			int y = (int)event.getY();

			x -= widget.getTotalPaddingLeft();
			y -= widget.getTotalPaddingTop();

			x += widget.getScrollX();
			y += widget.getScrollY();

			Layout layout = widget.getLayout();
			int line = layout.getLineForVertical(y);
			int offset = layout.getOffsetForHorizontal(line, x);

			ClickableSpan[] link = text.getSpans(offset, offset, ClickableSpan.class);

			if (link.length != 0) {
				if (action == MotionEvent.ACTION_UP) {
					link[0].onClick(widget);
				} else {
					Selection.setSelection(text, text.getSpanStart(link[0]), text.getSpanEnd(link[0]));
				}

				return true;
			}
		}

		return super.onTouchEvent(widget, text, event);
	}
}