package jp.deadend.noname.treenote;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class BookmarkManager extends ListActivity {
	static final String KEY_BOOKMARKS = "BookmarkMangerBookmarks";
	private BMAdapter mAdapter;
	private String[] mBookmarks;
	private List<Integer> mListData;
	private Stack<List<Integer>> mUndoBuffer;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		super.onCreate(savedInstanceState);
		setContentView(R.layout.bookmark_manager);

		Bundle extras = getIntent().getExtras();
		mBookmarks = null;
		if (extras == null) {
			finish();
		} else {
			mBookmarks = extras.getStringArray(KEY_BOOKMARKS);
		}
		if (mBookmarks == null) finish();

		mListData = new ArrayList<Integer>();
		for (int i=0; i < mBookmarks.length; i++) {
			mListData.add(Integer.valueOf(i));
		}
		mAdapter = new BMAdapter(this, mListData);
		setListAdapter(mAdapter);

		mUndoBuffer = new Stack<List<Integer>>();

		getListView().setOnItemSelectedListener(new OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				((ListView)parent).setItemChecked(position, true);
			}
			@Override
			public void onNothingSelected(AdapterView<?> parent) {}
		});

		findViewById(R.id.ButtonUp).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (moveUpSelected()) {
					ListView l = getListView();
					l.setItemChecked(l.getCheckedItemPosition()-1, true);
					mAdapter.updateItems(mListData);
				}
			}
		});
		findViewById(R.id.ButtonDown).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (moveDownSelected()) {
					ListView l = getListView();
					l.setItemChecked(l.getCheckedItemPosition()+1, true);
					mAdapter.updateItems(mListData);
				}
			}
		});
		findViewById(R.id.ButtonRemove).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (removeSelected()) {
					getListView().clearChoices();
					mAdapter.updateItems(mListData);
				}
			}
		});
		findViewById(R.id.ButtonUndo).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (!mUndoBuffer.empty()) {
					mListData = mUndoBuffer.pop();
					mAdapter.updateItems(mListData);
				}
			}
		});
		findViewById(R.id.ButtonOK).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				int size = mAdapter.getCount();
				int[] result = new int[size];
				for (int i=0; i<size; i++) {
					result[i] = mAdapter.getItem(i);
				}

				Intent intent = new Intent();
				intent.putExtra(KEY_BOOKMARKS, result);
				setResult(RESULT_OK, intent);

				finish();
			}
		});
		findViewById(R.id.ButtonCancel).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent intent = new Intent();
				setResult(RESULT_CANCELED, intent);
				finish();
			}
		});
	}

	private boolean moveDownSelected() {
		int size = mAdapter.getCount();
		int pos = getListView().getCheckedItemPosition();
		if (pos == -1 || pos >= size-1) {return false;}

		List<Integer> newList = new ArrayList<Integer>(size);
		for (int i=0; i<size; i++) {
			if (i == pos) {
				newList.add(mListData.get(i+1));
			} else if (i == pos+1) {
				newList.add(mListData.get(i-1));
			} else {
				newList.add(mListData.get(i));
			}
		}
		mUndoBuffer.push(new ArrayList<Integer>(mListData));
		mListData = newList;

		return true;
	}

	private boolean moveUpSelected() {
		int size = mAdapter.getCount();
		int pos = getListView().getCheckedItemPosition();
		if (pos <= 0) {return false;}

		List<Integer> newList = new ArrayList<Integer>(size);
		for (int i=0; i<size; i++) {
			if (i == pos-1) {
				newList.add(mListData.get(i+1));
			} else if (i == pos) {
				newList.add(mListData.get(i-1));
			} else {
				newList.add(mListData.get(i));
			}
		}
		mUndoBuffer.push(new ArrayList<Integer>(mListData));
		mListData = newList;

		return true;
	}

	private boolean removeSelected() {
		int size = mAdapter.getCount();
		int pos = getListView().getCheckedItemPosition();
		if (pos == -1) {return false;}

		List<Integer> newList = new ArrayList<Integer>(size-1);
		for (int i=0; i<size-1; i++) {
			if (i >= pos) {
				newList.add(mListData.get(i+1));
			} else {
				newList.add(mListData.get(i));
			}
		}
		mUndoBuffer.push(new ArrayList<Integer>(mListData));
		mListData = newList;

		return true;
	}

	private class BMAdapter extends ArrayAdapter<Integer> {
		private LayoutInflater mInflater;

		BMAdapter(Context context, List<Integer> items) {
			super(context, 0, items);
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		}

		void updateItems(List<Integer> items) {
			clear();
			for (int i=0; i<items.size(); i++) {add(items.get(i));}
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			SelectableTextView tv = (SelectableTextView)convertView;

			if (tv == null) {
				tv = (SelectableTextView)mInflater.inflate(R.layout.bookmark_manager_row, parent, false);
			}

			tv.setText(mBookmarks[getItem(position).intValue()]);

			return tv;
		}
	}
}