package jp.deadend.noname.treenote;

import com.mobidevelop.widget.SplitPaneLayout;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

public class FilteredTreeNote extends Activity {
	public static final String KEY_NODES	= "FilteredTreeNoteNodes";
	public static final String KEY_QUERY	= "FilteredTreeNoteQuery";
	private ListView mListView;
	private TextView mTextView;
	private String mQueryWord;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.filtered);
		mListView = (ListView)findViewById(R.id.MainListView);
		mTextView = (TextView)findViewById(R.id.ContentTextView);

		Intent it = getIntent();
		if (it != null) {
			@SuppressWarnings("unchecked")
			ArrayList<TreeNoteNodeInfo> nodes = (ArrayList<TreeNoteNodeInfo>)(it.getSerializableExtra(KEY_NODES));
			mQueryWord = it.getStringExtra(KEY_QUERY);
			if (nodes == null || mQueryWord == null) {
				finish();
			}

			((TextView)findViewById(R.id.TitleTextView)).setText(getString(R.string.label_search_results) + " " + mQueryWord);
			TreeNoteNodeInfoAdapter adapter = new TreeNoteNodeInfoAdapter(this, nodes);
			mListView.setAdapter(adapter);
		} else {
			finish();
		}

		mListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				select(position);
			}
		});
	  	mListView.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				select(position);
			}
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
			}
		});
	}

	private void select(int position) {
		TreeNoteNodeInfoAdapter adapter = (TreeNoteNodeInfoAdapter)mListView.getAdapter();
		adapter.setSelected(position);
		mListView.invalidateViews();
		String cont = adapter.getItem(position).content;
		if (cont.contains(mQueryWord)) {
			mTextView.setText(TreeNoteAdapter.spanMatchedWords(cont, mQueryWord));
		} else {
			mTextView.setText(cont);
		}
	}

	@Override
	protected void onResume() {
		TreeNoteNodeInfoAdapter adapter = (TreeNoteNodeInfoAdapter)mListView.getAdapter();

		int fontSize = Integer.parseInt(TreeNotePrefs.getFontSize(this));
		adapter.setFontSize(fontSize);
		mTextView.setTextSize(fontSize);
		((TextView)findViewById(R.id.TitleTextView)).setTextSize(fontSize+2);

		int lsp = Integer.parseInt(TreeNotePrefs.getLineSpacing(this));
		adapter.setLineSpacing(lsp);

		SplitPaneLayout spl = (SplitPaneLayout)findViewById(R.id.ContentSplitView);
		float spp = TreeNotePrefs.getContentSplitterPosition(this);
		if (spp >= 0) {
			spl.setSplitterPositionPercent(spp);
		}
		int sps = Integer.parseInt(TreeNotePrefs.getSplitterSize(this));
		spl.setSplitterSize(sps);

		super.onResume();
	}

	private class TreeNoteNodeInfoAdapter extends BaseAdapter {
		class ViewHolder {
			TextView hierarchyDescriptionView;
			TextView subjectView;
		}

		private List<TreeNoteNodeInfo> mList;
		private LayoutInflater mInflater;
		private int mSelected;
		private int mFontSize;
		private int mLineSpacing;
		private static final int SEARCH_HIGHLIGHT_COLOR = 0xFFFFBF00;

		TreeNoteNodeInfoAdapter(Context context, List<TreeNoteNodeInfo> items) {
			mList = items;
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			mSelected = -1;
		}

		void setFontSize(int size) {
			mFontSize = size;
		}

		void setLineSpacing(int lsp) {
			mLineSpacing = lsp;
		}

		void setSelected(int position) {
			mSelected = position;
		}

		void unSelect() {
			mSelected = -1;
		}

		@Override
		public int getCount() {
			return mList.size();
		}

		@Override
		public TreeNoteNodeInfo getItem(int position) {
			return mList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder holder;

			View view = convertView;
			if (view == null) {
				view = mInflater.inflate(R.layout.filtered_list_item, parent, false);

				TextView tv1 = (TextView)view.findViewById(R.id.HierarchyDescriptionView);
				TextView tv2 = (TextView)view.findViewById(R.id.SubjectView);

				holder = new ViewHolder();
				holder.hierarchyDescriptionView = tv1;
				holder.subjectView = tv2;

				view.setTag(holder);
    		} else {
				holder = (ViewHolder)view.getTag();
    		}

			holder.hierarchyDescriptionView.setTextSize(mFontSize);
			holder.hierarchyDescriptionView.setPadding(2, mLineSpacing, 4, mLineSpacing);
			holder.subjectView.setTextSize(mFontSize);
			holder.subjectView.setPadding(2, mLineSpacing, 4, mLineSpacing);

			holder.hierarchyDescriptionView.setText(mList.get(position).hierarchyDesc);
			String subj = mList.get(position).subject;
			if (subj.contains(mQueryWord)) {
				holder.subjectView.setText(TreeNoteAdapter.spanMatchedWords(subj, mQueryWord));
			} else {
				holder.subjectView.setText(subj);
			}

			if (position == mSelected) {
				holder.subjectView.setTextColor(0xFF000000);
				holder.subjectView.setBackgroundResource(R.drawable.list_selector_selected);
			} else {
				holder.subjectView.setTextColor(0xFFFFFFFF);
				holder.subjectView.setBackgroundResource(0);
			}

			return view;
		}
	}
}