package jp.deadend.noname.treenote;

import java.io.File;
import java.util.ArrayList;

import android.content.Intent;
import android.util.Log;
import android.view.Menu;
import android.view.Window;
import android.widget.LinearLayout;
import android.widget.TextView;

public class HoistedTreeNote extends TreeNote {
	private static final String BACKUP_SAVE_FILE_FOCUSED = "backup_focused.txt";
	static final String KEY_NODES = "HoistedTreeNoteNodes";
	static final String KEY_ROOTLEVEL = "HoistedTreeNoteRootLevel";
	static final String KEY_IDCOUNT = "HoistedTreeNoteIDCount";
	static final String KEY_ROOTID = "HoistedTreeNoteRootId";
	static final String KEY_ROOTSUBJ = "HoistedTreeNoteRootSubject";
	static final String KEY_COMMANDS = "HoistedTreeNoteCommands";
	static final String KEY_REDONUM = "HoistedTreeNoteRedoNum";

	private Integer mRootId;

	@Override
	void onCreateLocal() {
		Intent it = getIntent();
		if (it == null) {
			finish();
		}
		int rId = it.getIntExtra(KEY_ROOTID, -1);
		if (rId == -1) {
			mRootId = null;
		} else {
			mRootId = Integer.valueOf(rId);
		}

		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.main);

		TextView tv = new TextView(this);
		tv.setTextSize(Integer.parseInt(TreeNotePrefs.getFontSize(this))+2);
		tv.setTextColor(0xFF000000);
		tv.setBackgroundColor(0xFFD4D4D4);
		tv.setText(it.getStringExtra(KEY_ROOTSUBJ));
		((LinearLayout)findViewById(R.id.MainLayout)).addView(tv, 0);

		setupControl();
		setButtonEnabled(R.id.ButtonOpenFile, false);
		setButtonEnabled(R.id.ButtonSave, false);

		State state = (State)getLastNonConfigurationInstance();
		if (state == null) {
			File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE_FOCUSED);
			if (backupFile.canRead()) {
				//~ loadBackupFile();
				Log.e("TreeNote", "todo: recover from a backup on HoistedTreeNote.java");
				clearBackupSave();
				finish();
			} else {
				@SuppressWarnings("unchecked")
				ArrayList<TreeNoteNodeInfo> nodes = (ArrayList<TreeNoteNodeInfo>)(it.getSerializableExtra(KEY_NODES));
				int level = it.getIntExtra(KEY_ROOTLEVEL, -1);
				int count = it.getIntExtra(KEY_IDCOUNT, -1);
				if (nodes == null || level == -1 || count == -1) {
					finish();
				}
				mManager = TreeNoteStateManager.buildFromNodes(nodes, level, count);

				mAdapter = new TreeNoteAdapter(this, mManager, LEVEL_NUMBER, Integer.parseInt(TreeNotePrefs.getFontSize(this)));
				setAdapter(mAdapter);
				mAdapter.unSelect();

				setUndoRedoButtons();
			}
		} else {
			mManager = state.manager;
			int fontsize = Integer.parseInt(TreeNotePrefs.getFontSize(this));
			mAdapter = new TreeNoteAdapter(this, mManager, LEVEL_NUMBER, fontsize);
			setAdapter(mAdapter);
			if (state.selectedId != null) {
				select(state.selectedId);
			}

			setUndoRedoButtons();
		}
	}

	@Override
	public Object onRetainNonConfigurationInstance() {
		updateContent();
		State state = new State(mManager, null, null, mAdapter.getSelectedId(), null);

		clearBackupSave();

		return state;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.findItem(R.id.MenuFile).setEnabled(false);

		return true;
	}

	@Override
	public void performCommand(int cmd) {
		switch (cmd) {
		case COMMAND_SAVE:
		case COMMAND_SAVE_AS:
		case COMMAND_OPEN:
		case COMMAND_OPEN_RECENT:
		case COMMAND_NEWFILE:
		case COMMAND_ADD_BOOKMARK:
		case COMMAND_BOOKMARKS:
		case COMMAND_MANAGE_BOOKMARKS:
		case COMMAND_BACK:
			return;
		case COMMAND_QUIT:
			Intent intent = new Intent();

			if (!mManager.isUndoable() && !mManager.isRedoable()) {
				setResult(RESULT_CANCELED, intent);
			} else {
				intent.putExtra(KEY_COMMANDS, mManager.convertUndoRedoBufferInHoisted(mRootId));
				intent.putExtra(KEY_REDONUM, mManager.getRedoNum());
				setResult(RESULT_OK, intent);
			}

			finish();
		default:
			super.performCommand(cmd);
		}
	}

	@Override
	void backupSave() {
		if (!isFinishing()) {
			updateContent();
			File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE_FOCUSED);
			String indentStr = TreeNotePrefs.getIndentStr(this);
			String encoding = TreeNotePrefs.getEncoding(this);
			String newline = TreeNotePrefs.getNewLineChar(this);
			mManager.saveFile(backupFile, indentStr, encoding, newline);
		}
	}

	@Override
	void clearBackupSave() {
		File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE_FOCUSED);
		if (backupFile.canRead()) {
			backupFile.delete();
		}
	}
}