package jp.deadend.noname.treenote;

import android.content.Context;
import android.util.AttributeSet;
import android.util.SparseIntArray;
import android.view.KeyEvent;
import android.widget.LinearLayout;

public class LinearLayoutKeyDispatcher extends LinearLayout {
	interface Callback {
		void performCommand(int cmd);
		void openOptionsMenu();
	}

	private Callback mCallback = null;
	private boolean isModKeyPressed = false;
	private boolean isModKeyUsed = true;
	private SparseIntArray mShortcuts;

	enum ModKey {
		ALT_LEFT(KeyEvent.META_ALT_LEFT_ON),
		ALT_RIGHT(KeyEvent.META_ALT_RIGHT_ON),
		CTRL_LEFT(0x00002000),
		CTRL_RIGHT(0x00004000),
		META_LEFT(0x00020000),
		META_RIGHT(0x00040000),
		SHIFT_LEFT(KeyEvent.META_SHIFT_LEFT_ON),
		SHIFT_RIGHT(KeyEvent.META_SHIFT_RIGHT_ON),
		SYM(KeyEvent.META_SYM_ON),
		NUM_LOCK(0x00200000),
		SCROLL_LOCK(0x00400000),
		CAPS_LOCK(0x00100000),
		FUNCTION(0x00000008),				// also dynabook az ctrl
		CTRL_LIFETOUCH_NOTE(0x00001100),	// lifetouch note ctrl
		FN_005SH_LEFT(0x00010010),			// 005sh left fn
		FN_005SH_RIGHT(0x00100010),			// 005sh right fn
		MANNER_L04C(93),					// l-04c "manner", needs special treatment
		MENU(KeyEvent.KEYCODE_MENU),		// "menu", needs special treatment
		;

		private int mask;
		private ModKey(int m) {
			this.mask = m;
		}

		int getMask() {
			return this.mask;
		}
	}
	private ModKey mShortcutModKey = ModKey.ALT_LEFT;

    public LinearLayoutKeyDispatcher(Context context) {
        super(context);
    }

    public LinearLayoutKeyDispatcher(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

	public void setCallback(Callback c) {
		mCallback = c;
	}

	public void setModKey(String m) {
		try {
			mShortcutModKey = ModKey.valueOf(m);
		} catch (Exception e) {
		}
	}

	public void setShortcutKeys(SparseIntArray shortcuts) {
		mShortcuts = shortcuts;
	}

	public void reset() {
		isModKeyPressed = false;
		isModKeyUsed = true;
	}

	@Override
	public boolean dispatchKeyEventPreIme(KeyEvent event) {
		int keycode = event.getKeyCode();
		switch (mShortcutModKey) {
		case MENU:
		case MANNER_L04C:
			// these keys don't provide a metastate
			if (keycode == mShortcutModKey.getMask()) {
				if (event.getAction() == KeyEvent.ACTION_DOWN) {
					isModKeyPressed = true;
					isModKeyUsed = false;
					return true;
				} else if (event.getAction() == KeyEvent.ACTION_UP) {
					isModKeyPressed = false;
					if (mShortcutModKey == ModKey.MENU && !isModKeyUsed) {
						mCallback.openOptionsMenu();
					}
					isModKeyUsed = true;
					return true;
				}
			} else if (event.getAction() == KeyEvent.ACTION_DOWN && isModKeyPressed) {
				int cmd = mShortcuts.get(keycode, -1);
				if (cmd != -1 && mCallback != null) {
					mCallback.performCommand(cmd);
					isModKeyUsed = true;
					return true;
				}
			}
			break;
		default:
			int masked_meta_state = event.getMetaState() & mShortcutModKey.getMask();
			if (event.getAction() == KeyEvent.ACTION_DOWN && masked_meta_state != 0) {
				int cmd = mShortcuts.get(keycode, -1);
				if (cmd != -1 && mCallback != null) {
					mCallback.performCommand(cmd);
					return true;
				}
			}
			break;
		}

		return super.dispatchKeyEventPreIme(event);
	}
}