package jp.deadend.noname.treenote;

import pl.polidea.treeview.TreeViewList;
import pl.polidea.treeview.InMemoryTreeStateEditInfo;
import com.mobidevelop.widget.SplitPaneLayout;

import java.io.File;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.text.ClipboardManager;
import android.text.DynamicLayout;
import android.text.InputFilter;
import android.text.Layout;
import android.text.Spanned;
import android.text.util.Linkify;
import android.util.Log;
import android.util.SparseIntArray;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.HorizontalScrollView;
import android.widget.ListAdapter;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import android.widget.Toast;

public class TreeNote extends Activity implements LinearLayoutKeyDispatcher.Callback {
	TreeNoteStateManager mManager = null;
	private TreeNoteStateManager mManagerToSave = null;
	private TreeViewList mTreeView;
	//~ private EditText mContentEditText;
	private EditTextCursorWatcher mContentEditText;
	private EditText mAddNewEditText;
	private SplitPaneLayout mContentSplitView;
	private SplitPaneLayout mAddNewSplitView;
	TreeNoteAdapter mAdapter; // use also in HoistedTreeNote
	private TreeNoteClipboardNode mClipboard = null;
	private LinkedList<String> mRecentlyUsedFiles = null;
	private String mDirName = null;
	private String mFileName = null;
	private boolean mDontDim = false;
	private View mToolbar = null;
	private View mToolbar_mid = null;
	private View mToolbar_large = null;
	private Integer mReturnID = null;
	private Integer mSplitID = null;
	private int[] mSplitPos = new int[4];

	static final int LEVEL_NUMBER = 11;
	private static final int MAX_RECENTLY_USED = 10;
	private static final String BACKUP_SAVE_FILE = "backup.txt";
	private static final int DIALOG_EDIT			= 0;
	private static final int DIALOG_SEARCH			= 1;
	private static final int DIALOG_REPLACE			= 2;
	private static final int DIALOG_CONFIRM_SAVE	= 3;
	private static final int DIALOG_RECENTLY_USED	= 4;
	private static final int DIALOG_UI_SETTING		= 5;
	private static final int DIALOG_CREATE_SHORTCUT	= 6;
	private static final int DIALOG_BOOKMARKS		= 7;
	private static final int DIALOG_SPLIT_SUBJECT	= 8;
	private static final int DIALOG_SPLIT_CONTENT	= 9;
	private static final int REQUEST_OPEN_FILE		= 0;
	private static final int REQUEST_SAVE_AS		= 1;
	private static final int REQUEST_EDIT_CONTENT	= 2;
	private static final int REQUEST_MANAGE_BOOKMARKS = 3;
	private static final int REQUEST_HOIST			= 4;
	static final int COMMAND_QUIT			= 0;
	static final int COMMAND_TRACE			= 1;
	static final int COMMAND_ADDNODE		= 2;
	static final int COMMAND_PROMOTE		= 3;
	static final int COMMAND_DEMOTE			= 4;
	static final int COMMAND_MOVEUP			= 5;
	static final int COMMAND_MOVEDOWN		= 6;
	static final int COMMAND_CUT			= 7;
	static final int COMMAND_COPY			= 8;
	static final int COMMAND_PASTE			= 9;
	static final int COMMAND_SEARCH			= 10;
	static final int COMMAND_SEARCH_NEXT	= 11;
	static final int COMMAND_SEARCH_PREV	= 12;
	static final int COMMAND_REPLACE		= 13;
	static final int COMMAND_TOGGLE_TWO_PANE= 14;
	static final int COMMAND_TOGGLE_NEW_ITEM= 15;
	static final int COMMAND_TOGGLE_TOOLBAR	= 16;
	static final int COMMAND_SWITCH			= 17;
	static final int COMMAND_SAVE			= 18;
	static final int COMMAND_SAVE_AS		= 19;
	static final int COMMAND_OPEN			= 20;
	static final int COMMAND_OPEN_RECENT	= 21;
	static final int COMMAND_NEWFILE		= 22;
	static final int COMMAND_EXPAND_ALL		= 23;
	static final int COMMAND_COLLAPSE_ALL	= 24;
	static final int COMMAND_EXPAND_BELOW	= 25;
	static final int COMMAND_EDIT_NODE		= 26;
	static final int COMMAND_CONTENT_EDITOR	= 27;
	static final int COMMAND_CREATE_SHORTCUT= 28;
	static final int COMMAND_ADD_BOOKMARK	= 29;
	static final int COMMAND_BOOKMARKS		= 30;
	static final int COMMAND_MANAGE_BOOKMARKS= 31;
	static final int COMMAND_BACK			= 32;
	static final int COMMAND_UNDO			= 33;
	static final int COMMAND_REDO			= 34;
	static final int COMMAND_SHARE			= 35;
	static final int COMMAND_HOIST			= 36;
	static final int COMMAND_FILTER			= 37;
	static final int COMMAND_MERGE_TO_PREV	= 38;
	static final int COMMAND_MERGE_TO_NEXT	= 39;
	static final int COMMAND_SPLIT			= 40;

	class State {
		TreeNoteStateManager manager;
		String dirName;
		String fileName;
		Integer selectedId;
		LinkedList<String> recentlyUsedFiles;

		State(TreeNoteStateManager m, String d, String f, Integer id, LinkedList<String> ru) {
			manager = m;
			dirName = d;
			fileName = f;
			selectedId = id;
			recentlyUsedFiles = ru;
		}
	}

	interface DoAfterConfirmSave {
		void doProcess();
	}
	private DoAfterConfirmSave mDoAfterConfirmSave = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		onCreateLocal();
	}

	void onCreateLocal() {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.main);

		setupControl();

		State state = (State)getLastNonConfigurationInstance();
		if (state == null) {
			mRecentlyUsedFiles = new LinkedList<String>();

			Intent it = getIntent();
			String action = it.getAction();
			File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE);
			if (action.equals(Intent.ACTION_VIEW) || action.equals(Intent.ACTION_EDIT)) {
				Uri uri = getIntent().getData();
				if (uri.getScheme().equals("file")) {
					if (!loadFile(new File(uri.getPath()))) {
						newFile();
					}
				} else {
					loadLastFileOrCreateNew();
				}
			} else if (backupFile.canRead()) {
				loadBackupFile();
			} else {
				loadLastFileOrCreateNew();
			}
		} else {
			mRecentlyUsedFiles = state.recentlyUsedFiles;
			mDirName = state.dirName;
			mFileName = state.fileName;
			mManager = state.manager;
			int fontsize = Integer.parseInt(TreeNotePrefs.getFontSize(this));
			mAdapter = new TreeNoteAdapter(this, mManager, LEVEL_NUMBER, fontsize);
			setAdapter(mAdapter);
			if (state.selectedId != null) {
				select(state.selectedId);
			}
			setUndoRedoButtons();
		}
	}

	private void revertExpansionState() {
		String state = TreeNotePrefs.getLastExpansionState(this);
		if (state != null) {
			mManager.collapseChildren(null);
			mManager.expandAsSaved(null, state);

			int lastPos = TreeNotePrefs.getLastPosition(this);
			List<Integer> visibleList = mManager.getVisibleList();
			if (lastPos >= 0 && lastPos < visibleList.size()) {
				select(visibleList.get(lastPos));
			}
		}
	}

	private void loadLastFileOrCreateNew() {
		mDirName = TreeNotePrefs.getLastDir(this);
		mFileName = TreeNotePrefs.getLastFile(this);
		if (mDirName != null && mFileName != null) {
			if (loadFile(new File(mDirName + "/" + mFileName))) {
				revertExpansionState();
			} else {
				newFile();
			}
		} else {
			newFile();
		}
	}

	private void loadBackupFile() {
		mDirName = TreeNotePrefs.getLastDir(this);
		mFileName = TreeNotePrefs.getLastFile(this);

		File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE);
		String indentStr = TreeNotePrefs.getIndentStr(this);
		String encoding = TreeNotePrefs.getEncoding(this);
		TreeNoteStateManager manager = TreeNoteStateManager.loadFile(backupFile, indentStr, encoding, TreeNoteStateManager.getBookmarksFile(getFilesDir(), mDirName + "/" + mFileName));
		if (manager == null) {
			Toast.makeText(TreeNote.this, getString(R.string.error_backup_load_failed), Toast.LENGTH_SHORT).show();
			loadLastFileOrCreateNew();
		} else {
			mManager = manager;
			int fontsize = Integer.parseInt(TreeNotePrefs.getFontSize(this));
			mAdapter = new TreeNoteAdapter(this, mManager, LEVEL_NUMBER, fontsize);
			setAdapter(mAdapter);
			revertExpansionState();

			backupFile.delete();
		}
	}

	private void setShortcutKeys() {
        SparseIntArray shortcuts = new SparseIntArray();
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_promote)),		COMMAND_PROMOTE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_demote)),			COMMAND_DEMOTE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_moveup)),			COMMAND_MOVEUP);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_movedown)),		COMMAND_MOVEDOWN);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_mergeprev)),		COMMAND_MERGE_TO_PREV);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_mergenext)),		COMMAND_MERGE_TO_NEXT);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_split)),			COMMAND_SPLIT);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_cut)),			COMMAND_CUT);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_copy)),			COMMAND_COPY);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_paste)),			COMMAND_PASTE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_undo)),			COMMAND_UNDO);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_redo)),			COMMAND_REDO);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_switch)),			COMMAND_SWITCH);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_save)),			COMMAND_SAVE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_new)),			COMMAND_NEWFILE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_open)),			COMMAND_OPEN);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_editnode)),		COMMAND_EDIT_NODE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_editcontent)),	COMMAND_CONTENT_EDITOR);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_togglecontent)),	COMMAND_TOGGLE_TWO_PANE);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_togglenew)), 		COMMAND_TOGGLE_NEW_ITEM);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_searchnext)), 	COMMAND_SEARCH_NEXT);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_searchprev)), 	COMMAND_SEARCH_PREV);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_addbookmark)), 	COMMAND_ADD_BOOKMARK);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_bookmarks)), 	COMMAND_BOOKMARKS);
        shortcuts.put(TreeNotePrefs.getShortcut(this, getString(R.string.pref_key_back)), 	COMMAND_BACK);
        //~ shortcuts.put(KeyEvent.KEYCODE_DEL, COMMAND_TRACE);

        LinearLayoutKeyDispatcher llkd = (LinearLayoutKeyDispatcher)findViewById(R.id.MainLayout);
        llkd.setCallback(this);
        llkd.setModKey(TreeNotePrefs.getModKey(this));
        llkd.setShortcutKeys(shortcuts);
        llkd.reset();
	}

	private void setOnClickCommandToToolbar(int viewId, int command) {
		final int cmd = command;
		mToolbar.findViewById(viewId).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				performCommand(cmd);
			}
		});
		mToolbar_mid.findViewById(viewId).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				performCommand(cmd);
			}
		});
		mToolbar_large.findViewById(viewId).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				performCommand(cmd);
			}
		});
	}

	void setupControl() {
		mTreeView = (TreeViewList)findViewById(R.id.MainTreeView);
		mTreeView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			@Override
			public void onFocusChange(View view, boolean b) {
				if (b == true) {
					mTreeView.setSelection(mManager.getPosition(mAdapter.getSelectedId()));
				}
			}
		});
		mContentEditText = (EditTextCursorWatcher)findViewById(R.id.ContentEditText);
		mContentEditText.setCallback((VerticalSeekBar_Reverse)findViewById(R.id.ContentSeekBar));
		mContentEditText.setAutoLinkMask(Linkify.EMAIL_ADDRESSES|Linkify.MAP_ADDRESSES|Linkify.WEB_URLS);
		mContentEditText.setMovementMethod(ArrowKeyAndLinkMovementMethod.getInstance());
		mAddNewEditText = (EditText)findViewById(R.id.AddNewEditText);
		mContentSplitView = (SplitPaneLayout)findViewById(R.id.ContentSplitView);
		mAddNewSplitView = (SplitPaneLayout)findViewById(R.id.AddNewSplitView);

		mAddNewEditText.setOnKeyListener(new View.OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (keyCode == KeyEvent.KEYCODE_ENTER) {
					if (event.getAction() == KeyEvent.ACTION_UP) {
						performCommand(COMMAND_ADDNODE);
					}
					return true;
				}
				return false;
			}
		});

		findViewById(R.id.ButtonNew).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				performCommand(COMMAND_ADDNODE);
			}
		});

		mToolbar		= getLayoutInflater().inflate(R.layout.toolbar, null);
		mToolbar_mid	= getLayoutInflater().inflate(R.layout.toolbar_mid, null);
		mToolbar_large	= getLayoutInflater().inflate(R.layout.toolbar_large, null);
		setOnClickCommandToToolbar(R.id.ButtonPromote,		COMMAND_PROMOTE);
		setOnClickCommandToToolbar(R.id.ButtonDemote,		COMMAND_DEMOTE);
		setOnClickCommandToToolbar(R.id.ButtonMoveUp,		COMMAND_MOVEUP);
		setOnClickCommandToToolbar(R.id.ButtonMoveDown,		COMMAND_MOVEDOWN);
		setOnClickCommandToToolbar(R.id.ButtonMergeToPrev,	COMMAND_MERGE_TO_PREV);
		setOnClickCommandToToolbar(R.id.ButtonMergeToNext,	COMMAND_MERGE_TO_NEXT);
		setOnClickCommandToToolbar(R.id.ButtonSplit,		COMMAND_SPLIT);
		setOnClickCommandToToolbar(R.id.ButtonCut,			COMMAND_CUT);
		setOnClickCommandToToolbar(R.id.ButtonCopy,			COMMAND_COPY);
		setOnClickCommandToToolbar(R.id.ButtonPaste,		COMMAND_PASTE);
		setOnClickCommandToToolbar(R.id.ButtonSearch,		COMMAND_SEARCH);
		setOnClickCommandToToolbar(R.id.ButtonSearchNext,	COMMAND_SEARCH_NEXT);
		setOnClickCommandToToolbar(R.id.ButtonSearchPrev,	COMMAND_SEARCH_PREV);
		setOnClickCommandToToolbar(R.id.ButtonReplace,		COMMAND_REPLACE);
		setOnClickCommandToToolbar(R.id.ButtonBookmarks,	COMMAND_BOOKMARKS);
		setOnClickCommandToToolbar(R.id.ButtonAddBookmark,	COMMAND_ADD_BOOKMARK);
		setOnClickCommandToToolbar(R.id.ButtonBack,			COMMAND_BACK);
		setOnClickCommandToToolbar(R.id.ButtonOpenFile,		COMMAND_OPEN);
		setOnClickCommandToToolbar(R.id.ButtonSave,			COMMAND_SAVE);
		setOnClickCommandToToolbar(R.id.ButtonUndo,			COMMAND_UNDO);
		setOnClickCommandToToolbar(R.id.ButtonRedo,			COMMAND_REDO);
		setOnClickCommandToToolbar(R.id.ButtonShare,		COMMAND_SHARE);

		((VerticalSeekBar_Reverse)findViewById(R.id.ContentSeekBar)).setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {}
			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {}
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
				if (fromUser) {
					int offset = (mContentEditText.length()) * progress / 100;
					Layout layout = mContentEditText.getLayout();
					if (layout != null) {
						int line = layout.getLineForOffset(offset);
						int y = layout.getLineTop(line);
						//~ mContentEditText.scrollTo(0, y);
						//~ mContentEditText.moveCursorToVisibleOffset();
						mContentEditText.setSelection(((DynamicLayout)layout).getLineStart(line));
					}
				}
			}
		});
	}

	void setButtonEnabled(int buttonId, boolean enabled) {
		mToolbar.findViewById(buttonId).setEnabled(enabled);
		mToolbar_mid.findViewById(buttonId).setEnabled(enabled);
		mToolbar_large.findViewById(buttonId).setEnabled(enabled);
	}

	private void setupToolbar(String toolbar, String size) {
		HorizontalScrollView scv1 = (HorizontalScrollView)findViewById(R.id.ToolbarTop);
		HorizontalScrollView scv2 = (HorizontalScrollView)findViewById(R.id.ToolbarBottom);
		View tb;
		if (size.equals(getString(R.string.pref_value_toolbarsize_large))) {
			tb = mToolbar_large;
		} else if (size.equals(getString(R.string.pref_value_toolbarsize_medium))) {
			tb = mToolbar_mid;
		} else {
			tb = mToolbar;
		}

		scv1.removeAllViews();
		scv2.removeAllViews();
		if (toolbar.equals(getString(R.string.pref_value_toolbar_top))) {
			scv1.addView(tb);
			scv1.setFocusable(true);
			scv2.setFocusable(false);
		} else if (toolbar.equals(getString(R.string.pref_value_toolbar_bottom))) {
			scv2.addView(tb);
			scv2.setFocusable(true);
			scv1.setFocusable(false);
		} else {
			scv1.setFocusable(false);
			scv2.setFocusable(false);
		}

		if (mReturnID == null) {
			tb.findViewById(R.id.ButtonBack).setEnabled(false);
		} else {
			tb.findViewById(R.id.ButtonBack).setEnabled(true);
		}

		if (mManager.getBookmarksSize() == 0) {
			tb.findViewById(R.id.ButtonBookmarks).setEnabled(false);
		} else {
			tb.findViewById(R.id.ButtonBookmarks).setEnabled(true);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();

		clearBackupSave();

		setShortcutKeys();
		int fontsize = Integer.parseInt(TreeNotePrefs.getFontSize(this));
		mContentEditText.setTextSize(fontsize);
		mAddNewEditText.setTextSize(fontsize);
		int lsp = Integer.parseInt(TreeNotePrefs.getLineSpacing(this));
		boolean sl = TreeNotePrefs.getIsSingleLineMode(this);
		mAdapter.setFontSize(fontsize);
		mAdapter.setLineSpacing(lsp);
		mAdapter.setSingleLineMode(sl);
		mAdapter.refresh();

		if (TreeNotePrefs.getAutoHideButtons(this)) {
			if (getResources().getConfiguration().hardKeyboardHidden == Configuration.HARDKEYBOARDHIDDEN_NO) {
				setupToolbar(getString(R.string.pref_value_toolbarsize_medium), getString(R.string.pref_value_toolbar_none));
				findViewById(R.id.ButtonNew).setVisibility(View.GONE);
			} else {
				setupToolbar(TreeNotePrefs.getToolbar(this), TreeNotePrefs.getToolbarSize(this));
			}
		} else {
			setupToolbar(TreeNotePrefs.getToolbar(this), TreeNotePrefs.getToolbarSize(this));
		}

		if (TreeNotePrefs.getSeekbar(this)) {
			findViewById(R.id.ContentSeekBar).setVisibility(View.VISIBLE);
		} else {
			findViewById(R.id.ContentSeekBar).setVisibility(View.GONE);
		}
		float spp_new = TreeNotePrefs.getNewTextSplitterPosition(this);
		float spp_content = TreeNotePrefs.getContentSplitterPosition(this);
		if (spp_new >= 0) {
			mAddNewSplitView.setSplitterPositionPercent(spp_new);
		}
		if (spp_content >= 0) {
			mContentSplitView.setSplitterPositionPercent(spp_content);
		}
		int spl_size = Integer.parseInt(TreeNotePrefs.getSplitterSize(this));
		mAddNewSplitView.setSplitterSize(spl_size);
		mContentSplitView.setSplitterSize(spl_size);

		mDontDim = TreeNotePrefs.getDontDim(this);

		mManager.registerActivity(this);
	}

	@Override
	protected void onPause() {
		float spp_new = mAddNewSplitView.getSplitterPositionPercent();
		float spp_content = mContentSplitView.getSplitterPositionPercent();

		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
		pref.edit()
			.putString(getString(R.string.pref_key_lastdir), mDirName)
			.putString(getString(R.string.pref_key_lastfile), mFileName)
			.putString(getString(R.string.pref_key_lastexpansionstate), mManager.listExpandedNodes())
			.putInt(getString(R.string.pref_key_lastposition), mManager.getPosition(mAdapter.getSelectedId()))
			.putFloat(getString(R.string.pref_key_splitternew), spp_new)
			.putFloat(getString(R.string.pref_key_splittercontent), spp_content)
			.putBoolean(getString(R.string.pref_key_issingleline), mAdapter.getSingleLineMode())
			.commit();

		mManager.unregisterActivity();

		if (mManager.isUndoable()) {
			backupSave();
		} else {
			clearBackupSave();
		}

		mManager.saveBookmarks(TreeNoteStateManager.getBookmarksFile(getFilesDir(), mDirName + "/" + mFileName));

		super.onPause();
	}

	void backupSave() {
		updateContent();
		File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE);
		String indentStr = TreeNotePrefs.getIndentStr(this);
		String encoding = TreeNotePrefs.getEncoding(this);
		String newline = TreeNotePrefs.getNewLineChar(this);
		mManager.saveFile(backupFile, indentStr, encoding, newline);
	}

	void clearBackupSave() {
		File backupFile = new File(getFilesDir(), BACKUP_SAVE_FILE);
		if (backupFile.canRead()) {
			backupFile.delete();
		}
	}

	@Override
	public Object onRetainNonConfigurationInstance() {
		updateContent();
		State state = new State(mManager, mDirName, mFileName, mAdapter.getSelectedId(), mRecentlyUsedFiles);

		clearBackupSave();

		return state;
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		int action = event.getAction();
		int keycode = event.getKeyCode();
		if (action == KeyEvent.ACTION_DOWN && keycode == KeyEvent.KEYCODE_BACK) {
			performCommand(COMMAND_QUIT);
			return true;
		} else if (action == KeyEvent.ACTION_UP && keycode == KeyEvent.KEYCODE_SEARCH) {
			performCommand(COMMAND_SEARCH);
			return true;
		}

		return super.dispatchKeyEvent(event);
	}

	void setAdapter(ListAdapter adapter) {
		mTreeView.setAdapter(adapter);
	}

	private void prepareNewManager() {
		mAdapter = new TreeNoteAdapter(this, mManager, LEVEL_NUMBER, Integer.parseInt(TreeNotePrefs.getFontSize(this)));
		setAdapter(mAdapter);
		mAdapter.unSelect();

		setButtonEnabled(R.id.ButtonUndo, false);
		setButtonEnabled(R.id.ButtonRedo, false);

		mAddNewEditText.requestFocus();
	}

	public void performCommand(int cmd) {
		Integer id = mAdapter.getSelectedId();
		Integer addedId = null;
		View focused = getWindow().getCurrentFocus();
		ClipboardManager cm;

		switch (cmd) {
		case COMMAND_ADDNODE:
			String str = mAddNewEditText.getText().toString().replace("\n", "");
			if (str.length() == 0) return;

			if (mManager.commitCommand(mManager.editCommand_addAfterChild(str, id))) {
				select(mManager.getLastID());
				mAddNewEditText.setText("");
				setUndoRedoButtons();
			}
			break;
		case COMMAND_PROMOTE:
			if (mManager.commitCommand(mManager.editCommand_promote(id))) {
				setUndoRedoButtons();
			}
			break;
		case COMMAND_DEMOTE:
			if (mManager.commitCommand(mManager.editCommand_demote(id))) {
				setUndoRedoButtons();
			}
			break;
		case COMMAND_MOVEUP:
			if (mManager.commitCommand(mManager.editCommand_moveUp(id))) {
				setUndoRedoButtons();
				scrollTo(id);
			}
			break;
		case COMMAND_MOVEDOWN:
			if (mManager.commitCommand(mManager.editCommand_moveDown(id))) {
				setUndoRedoButtons();
				scrollTo(id);
			}
			break;
		case COMMAND_CUT:
			if (focused.equals(mAddNewEditText) || focused.equals(mContentEditText)) {
				cm = (ClipboardManager)getSystemService(CLIPBOARD_SERVICE);
				EditTextUtils.textCut((EditText)focused, cm);
			} else if (focused.equals(mTreeView)) {
				if (id != null) {
					updateContent();
					mClipboard = mManager.readToClipboard(id);
					if (mManager.commitCommand(mManager.editCommand_removeNodeRecursively(id))) {
						setUndoRedoButtons();
						mAdapter.unSelect();
					}
				}
			}
			break;
		case COMMAND_COPY:
			if (focused.equals(mAddNewEditText) || focused.equals(mContentEditText)) {
				cm = (ClipboardManager)getSystemService(CLIPBOARD_SERVICE);
				EditTextUtils.textCopy((EditText)focused, cm);
			} else if (focused.equals(mTreeView)) {
				if (id != null) {
					updateContent();
					mClipboard = mManager.readToClipboard(id);
				}
			}
			break;
		case COMMAND_PASTE:
			if (focused.equals(mAddNewEditText) || focused.equals(mContentEditText)) {
				cm = (ClipboardManager)getSystemService(CLIPBOARD_SERVICE);
				EditTextUtils.textPaste((EditText)focused, cm);
			} else if (focused.equals(mTreeView)) {
				if (id != null) {
					List<InMemoryTreeStateEditInfo<Integer>> pasteCmd = mManager.editCommand_pasteFromClipboard(mClipboard, id);
					addedId = ((TreeNoteEditInfo)pasteCmd.get(0)).getId();
					if (mManager.commitCommand(pasteCmd)) {
						select(addedId);
						setUndoRedoButtons();
					}
				}
			}
			break;
		case COMMAND_SEARCH:
			removeDialog(DIALOG_SEARCH);
			showDialog(DIALOG_SEARCH);
			break;
		case COMMAND_SEARCH_NEXT:
		case COMMAND_SEARCH_PREV:
			String query = mAdapter.getQueryWord();
			List<Integer> matchedId = mManager.search(query);
			if (matchedId.size() == 0) {
				Toast.makeText(TreeNote.this, getString(R.string.error_no_query), Toast.LENGTH_SHORT).show();
				return;
			}

			List<Integer> idlist = mManager.getIdList();
			int next_idx = 0;
			if (id != null) {
				next_idx = idlist.indexOf(id);
			} else {
				if (cmd == COMMAND_SEARCH_NEXT) {
					next_idx = -1;
				} else if (cmd == COMMAND_SEARCH_PREV) {
					next_idx = idlist.size();
				}
			}
			do {
				if (cmd == COMMAND_SEARCH_NEXT) {
					next_idx++;
				} else if (cmd == COMMAND_SEARCH_PREV) {
					next_idx--;
				} else {
					Toast.makeText(TreeNote.this, getString(R.string.error_unknown), Toast.LENGTH_SHORT).show();
					return;
				}
				if (next_idx >= idlist.size() || next_idx < 0) {
					Toast.makeText(TreeNote.this, getString(R.string.error_search_not_found, query), Toast.LENGTH_SHORT).show();
					return;
				}
			} while (!matchedId.contains(idlist.get(next_idx)));

			if (mReturnID == null) {setReturnID(id);}
			mManager.expandToId(idlist.get(next_idx));
			select(idlist.get(next_idx));
			break;
		case COMMAND_REPLACE:
			if (mAdapter.getQueryWord() == null) {
				Toast.makeText(TreeNote.this, getString(R.string.error_not_searched), Toast.LENGTH_SHORT).show();
				return;
			} else {
				removeDialog(DIALOG_REPLACE);
				showDialog(DIALOG_REPLACE);
			}
			break;
		case COMMAND_FILTER:
			if (mAdapter.getQueryWord() == null) {
				Toast.makeText(TreeNote.this, getString(R.string.error_not_searched), Toast.LENGTH_SHORT).show();
				return;
			} else {
				Intent filterIntent = new Intent(TreeNote.this, FilteredTreeNote.class);
				String queryWord = mAdapter.getQueryWord();
				filterIntent.putExtra(FilteredTreeNote.KEY_QUERY, queryWord);
				filterIntent.putExtra(FilteredTreeNote.KEY_NODES, mManager.listSearchedNodes(queryWord));
				startActivity(filterIntent);
			}
			break;
		case COMMAND_TOGGLE_TWO_PANE:
			mContentSplitView.toggleSubView();
			break;
		case COMMAND_TOGGLE_NEW_ITEM:
			mAddNewSplitView.toggleSubView();
			break;
		case COMMAND_SWITCH:
			if (focused.equals(mAddNewEditText)) {
				mTreeView.requestFocus();
			} else if (focused.equals(mTreeView)) {
				if (!mContentSplitView.isSubViewMinimized()) {
					mContentEditText.requestFocus();
 				} else if (!mAddNewSplitView.isSubViewMinimized()) {
					mAddNewEditText.requestFocus();
				}
			} else if (focused.equals(mContentEditText)) {
				if (!mAddNewSplitView.isSubViewMinimized()) {
					mAddNewEditText.requestFocus();
				} else {
					mTreeView.requestFocus();
				}
			}
			break;
		case COMMAND_SAVE:
			updateContent();
			mManagerToSave = mManager.clone();
			if (!saveToCurrentFile()) {
				saveAsActivity();
			} else if (mDoAfterConfirmSave != null) {
				mDoAfterConfirmSave.doProcess();
				mDoAfterConfirmSave = null;
			}
			break;
		case COMMAND_SAVE_AS:
			updateContent();
			mManagerToSave = mManager.clone();
			saveAsActivity();
			break;
		case COMMAND_OPEN:
			if (mManager.isUndoable()) {
				showConfirmSaveDialog(new DoAfterConfirmSave() {
					@Override
					public void doProcess() {
						openFileActivity();
					}
				});
			} else {
				openFileActivity();
			}
			break;
		case COMMAND_OPEN_RECENT:
			if (mManager.isUndoable()) {
				showConfirmSaveDialog(new DoAfterConfirmSave() {
					@Override
					public void doProcess() {
						removeDialog(DIALOG_RECENTLY_USED);
						showDialog(DIALOG_RECENTLY_USED);
					}
				});
			} else {
				removeDialog(DIALOG_RECENTLY_USED);
				showDialog(DIALOG_RECENTLY_USED);
			}
			break;
		case COMMAND_NEWFILE:
			if (mManager.isUndoable()) {
				showConfirmSaveDialog(new DoAfterConfirmSave() {
					@Override
					public void doProcess() {
						newFile();
						mManager.registerActivity(TreeNote.this);
					}
				});
			} else {
				newFile();
				mManager.registerActivity(TreeNote.this);
			}
			break;
		case COMMAND_EXPAND_ALL:
			mManager.expandEverythingBelow(null);
			break;
		case COMMAND_COLLAPSE_ALL:
			mManager.collapseChildren(null);
			break;
		case COMMAND_EXPAND_BELOW:
			mManager.expandEverythingBelow(id);
			break;
		case COMMAND_EDIT_NODE:
			removeDialog(TreeNote.DIALOG_EDIT);
			showDialog(TreeNote.DIALOG_EDIT);
			break;
		case COMMAND_CONTENT_EDITOR:
			if (id != null) {
				Intent intent = new Intent(this, TextEditor.class);
				intent.putExtra(TextEditor.KEY_STRING, mContentEditText.getText().toString());
				intent.putExtra(TextEditor.KEY_TITLE, mManager.getSubject(id));
				startActivityForResult(intent, REQUEST_EDIT_CONTENT);
			}
			break;
		case COMMAND_CREATE_SHORTCUT:
			if (mFileName == null || mDirName == null) {
				Toast.makeText(TreeNote.this, getString(R.string.error_filename_needed), Toast.LENGTH_SHORT).show();
			} else {
				showDialog(DIALOG_CREATE_SHORTCUT);
			}
			break;
		case COMMAND_ADD_BOOKMARK:
			if (id == null) {
				Toast.makeText(TreeNote.this, getString(R.string.error_not_selected), Toast.LENGTH_SHORT).show();
			} else {
				mManager.addBookmark(id);
				setButtonEnabled(R.id.ButtonBookmarks, true);
			}
			break;
		case COMMAND_BOOKMARKS:
			removeDialog(DIALOG_BOOKMARKS);
			showDialog(DIALOG_BOOKMARKS);
			break;
		case COMMAND_MANAGE_BOOKMARKS:
			List<Integer> bookmarks = mManager.getBookmarks();
			int b_size = bookmarks.size();
			String[] items = new String[b_size];
			for (int i=0; i<b_size; i++) {
				items[i] = mManager.getSubject(bookmarks.get(i));
			}

			Intent intent = new Intent(TreeNote.this, BookmarkManager.class);
			intent.putExtra(BookmarkManager.KEY_BOOKMARKS, items);
			startActivityForResult(intent, REQUEST_MANAGE_BOOKMARKS);
			break;
		case COMMAND_BACK:
			if (mReturnID != null) {select(mReturnID);}
			unsetReturnID();
			break;
		case COMMAND_UNDO:
			mManager.performUndo();
			setUndoRedoButtons();
			break;
		case COMMAND_REDO:
			mManager.performRedo();
			setUndoRedoButtons();
			break;
		case COMMAND_SHARE:
			if (id != null) {
				StringWriter sw = new StringWriter();
				if (!mManager.writeNode(sw, id, TreeNotePrefs.getIndentStr(this), "\n")) return;
				sw.flush();

				Intent shareIntent = new Intent(Intent.ACTION_SEND);
				shareIntent.setType("text/plain");
				shareIntent.putExtra(Intent.EXTRA_TEXT, sw.toString());
				try {
					startActivityForResult(shareIntent, 0);
				} catch (android.content.ActivityNotFoundException ex) {
					Toast.makeText(TreeNote.this, getString(R.string.error_activity_not_found), Toast.LENGTH_SHORT).show();
				}
			}
			break;
		case COMMAND_HOIST:
			if (id != null) {
				ArrayList<TreeNoteNodeInfo> nodes = new ArrayList<TreeNoteNodeInfo>();
				int level = mManager.getLevel(id);
				mManager.addToListBelowNode(nodes, id, level);
				Intent hoistIntent = new Intent(TreeNote.this, HoistedTreeNote.class);
				hoistIntent.putExtra(HoistedTreeNote.KEY_NODES, nodes);
				hoistIntent.putExtra(HoistedTreeNote.KEY_ROOTLEVEL, level);
				hoistIntent.putExtra(HoistedTreeNote.KEY_IDCOUNT, mManager.getNextIDCounter());
				hoistIntent.putExtra(HoistedTreeNote.KEY_ROOTID, id.intValue());
				hoistIntent.putExtra(HoistedTreeNote.KEY_ROOTSUBJ, mManager.getSubjectsFromRoot(id));
				startActivityForResult(hoistIntent, REQUEST_HOIST);
			}
			break;
		case COMMAND_MERGE_TO_PREV:
			Integer prevId = mManager.getPreviousSibling(id);
			if (mManager.commitCommand(mManager.editCommand_mergeToPrev(id))) {
				select(prevId);
				setUndoRedoButtons();
			}
			break;
		case COMMAND_MERGE_TO_NEXT:
			Integer nextId = mManager.getNextSibling(id);
			if (mManager.commitCommand(mManager.editCommand_mergeToNext(id))) {
				select(nextId);
				setUndoRedoButtons();
			}
			break;
		case COMMAND_SPLIT:
			mSplitID = mAdapter.getSelectedId();
			removeDialog(TreeNote.DIALOG_SPLIT_SUBJECT);
			showDialog(TreeNote.DIALOG_SPLIT_SUBJECT);
			break;
		case COMMAND_QUIT:
			if (mManager.isUndoable()) {
				showConfirmSaveDialog(new DoAfterConfirmSave() {
					@Override
					public void doProcess() {
						PreferenceManager.getDefaultSharedPreferences(TreeNote.this).edit()
							.putString(getString(R.string.pref_key_lastdir), mDirName)
							.putString(getString(R.string.pref_key_lastfile), mFileName)
							.commit();

						mManager.clearEditHistory(); // not to save backup
						finish();
					}
				});
			} else {
				finish();
			}
			break;
		case COMMAND_TRACE:
			Trace.d(mManager.toString());
			break;
		}
	}

	void setUndoRedoButtons() {
		setButtonEnabled(R.id.ButtonUndo, mManager.isUndoable());
		setButtonEnabled(R.id.ButtonRedo, mManager.isRedoable());
	}

	private boolean loadFile(File file) {
		if (!file.exists()) {
			Toast.makeText(TreeNote.this, getString(R.string.error_file_not_found, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
			return false;
		}
		if (!file.canRead()) {
			Toast.makeText(TreeNote.this, getString(R.string.error_read_protected, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
			return false;
		}

		String indentStr = TreeNotePrefs.getIndentStr(this);
		String encoding = TreeNotePrefs.getEncoding(this);
		TreeNoteStateManager manager = TreeNoteStateManager.loadFile(file, indentStr, encoding, TreeNoteStateManager.getBookmarksFile(getFilesDir(), file.getAbsolutePath()));
		if (manager == null) {
			Toast.makeText(TreeNote.this, getString(R.string.error_load_failed, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
			mFileName = null;
			return false;
		} else {
			mFileName = file.getName();
			mDirName = file.getParent();
			mManager = manager;
			prepareNewManager();
			if (TreeNotePrefs.getCollapsedOnLoad(this)) {
				mManager.collapseChildren(null);
			}
			updateRecentlyUsed(file.getAbsolutePath());
			return true;
		}
	}

	private void newFile() {
		mFileName = null;
		if (mDirName == null) {
			mDirName = Environment.getExternalStorageDirectory().getPath();
		}
		mManager = new TreeNoteStateManager();
		prepareNewManager();
	}

	private boolean saveToFile(File file) {
		if (file.exists()) {
			if (file.isDirectory()) {
				Toast.makeText(TreeNote.this, getString(R.string.error_isdirectory, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
				return false;
			} else if (!file.canWrite()) {
				Toast.makeText(TreeNote.this, getString(R.string.error_write_protected, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
				return false;
			}
		}

		updateContent();
		String indentStr = TreeNotePrefs.getIndentStr(this);
		String encoding = TreeNotePrefs.getEncoding(this);
		String newline = TreeNotePrefs.getNewLineChar(this);
		if (mManagerToSave.saveFile(file, indentStr, encoding, newline)) {
			Toast.makeText(TreeNote.this, getString(R.string.message_saved, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
			mFileName = file.getName();
			mDirName = file.getParent();
			mManagerToSave = null;
			mManager.clearEditHistory();
			setUndoRedoButtons();
			return true;
		} else {
			Toast.makeText(TreeNote.this, getString(R.string.error_not_saved, file.getAbsolutePath()), Toast.LENGTH_SHORT).show();
			return false;
		}
	}

	private boolean saveToCurrentFile() {
		if (mDirName == null || mFileName == null) return false;
		File file = new File(mDirName + "/" + mFileName);

		return saveToFile(file);
	}

	private void saveAsActivity() {
		Intent intent = new Intent(TreeNote.this, FileChooser.class);
		intent.putExtra(FileChooser.KEY_MODE, FileChooser.MODE_SAVE);
		intent.putExtra(FileChooser.KEY_DIRNAME, mDirName);
		startActivityForResult(intent, REQUEST_SAVE_AS);
	}

	private void openFileActivity() {
		Intent intent = new Intent(TreeNote.this, FileChooser.class);
		intent.putExtra(FileChooser.KEY_MODE, FileChooser.MODE_OPEN);
		intent.putExtra(FileChooser.KEY_DIRNAME, mDirName);
		intent.putExtra(FileChooser.KEY_FONTSIZE, Integer.parseInt(TreeNotePrefs.getFontSize(this)));
		startActivityForResult(intent, REQUEST_OPEN_FILE);
	}

	void updateContent() {
		Integer id = mAdapter.getSelectedId();
		if (id != null) {
			if (mManager.commitCommand(mManager.editCommand_changeContent(id, mContentEditText.getText().toString()))) {
				setUndoRedoButtons();
			}
		}
	}

	void refreshContent() {
		Integer id = mAdapter.getSelectedId();
		if (id != null) {
			mContentEditText.setText(mManager.getContent(id));
		}
	}

	void displayContent(CharSequence text) {
		mContentEditText.setText(text);
	}

	void setFocusToTreeView() {
		mTreeView.requestFocus();
	}

	void select(Integer id) {
		mAdapter.handleItemSelect(null, id);
		scrollTo(id);
	}

	void scrollTo(Integer id) {
		int position = mManager.getPosition(id);
		if (position < mTreeView.getFirstVisiblePosition() || mTreeView.getLastVisiblePosition() < position) {
			mTreeView.setSelection(position);
		}
	}

	private void setReturnID(Integer id) {
		if (id != null) {
			mReturnID = id;
			setButtonEnabled(R.id.ButtonBack, true);
		}
	}

	private void unsetReturnID() {
		mReturnID = null;
		setButtonEnabled(R.id.ButtonBack, false);
	}

	private void splitNode() {
		if (mManager.commitCommand(mManager.editCommand_split(mSplitID, mSplitPos[0], mSplitPos[1], mSplitPos[2], mSplitPos[3], getString(R.string.default_node_name)))) {
			setUndoRedoButtons();
			select(mManager.getLastID());
		}

		mSplitID = null;
		for (int i=0; i<4; i++) mSplitPos[i] = 0;
	}

	@Override
	public void openOptionsMenu() {
		super.openOptionsMenu();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main_menu, menu);

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.MenuOpen:
			performCommand(COMMAND_OPEN);
			break;
		case R.id.MenuNew:
			performCommand(COMMAND_NEWFILE);
			break;
		case R.id.MenuSave:
			performCommand(COMMAND_SAVE);
			break;
		case R.id.MenuSaveAs:
			performCommand(COMMAND_SAVE_AS);
			break;
		case R.id.MenuOpenRecent:
			performCommand(COMMAND_OPEN_RECENT);
			break;
		case R.id.MenuCreateShortcut:
			performCommand(COMMAND_CREATE_SHORTCUT);
			break;
		case R.id.MenuExpandAll:
			performCommand(COMMAND_EXPAND_ALL);
			break;
		case R.id.MenuCollapseAll:
			performCommand(COMMAND_COLLAPSE_ALL);
			break;
		case R.id.MenuExpandBelow:
			performCommand(COMMAND_EXPAND_BELOW);
			break;
		case R.id.MenuContentEditor:
			performCommand(COMMAND_CONTENT_EDITOR);
			break;
		case R.id.MenuPrefs:
			Intent intent = new Intent(this, TreeNotePrefs.class);
			startActivity(intent);
			break;
		case R.id.MenuDoSearch:
			performCommand(COMMAND_SEARCH);
			break;
		case R.id.MenuReplace:
			performCommand(COMMAND_REPLACE);
			break;
		case R.id.MenuSearchNext:
			performCommand(COMMAND_SEARCH_NEXT);
			break;
		case R.id.MenuSearchPrev:
			performCommand(COMMAND_SEARCH_PREV);
			break;
		case R.id.MenuSearchFilter:
			performCommand(COMMAND_FILTER);
			break;
		case R.id.MenuInterface:
			removeDialog(DIALOG_UI_SETTING);
			showDialog(DIALOG_UI_SETTING);
			break;
		case R.id.MenuShare:
			performCommand(COMMAND_SHARE);
			break;
		case R.id.MenuHoist:
			performCommand(COMMAND_HOIST);
			break;
		default:
			break;
		}

		return true;
	}

	private void updateRecentlyUsed(String newFile) {
		while (mRecentlyUsedFiles.contains(newFile)) {
			mRecentlyUsedFiles.remove(newFile);
		}
		mRecentlyUsedFiles.addFirst(newFile);

		while (mRecentlyUsedFiles.size() > MAX_RECENTLY_USED) {
			mRecentlyUsedFiles.removeLast();
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent) {
		String str = null;
		mManager.registerActivity(this);

		if (resultCode == RESULT_OK) {
			switch (requestCode) {
			case REQUEST_OPEN_FILE:
				str = intent.getStringExtra(FileChooser.KEY_FILEPATH);
				if (str == null) return;
				loadFile(new File(str));
				break;
			case REQUEST_SAVE_AS:
				str = intent.getStringExtra(FileChooser.KEY_FILEPATH);
				if (str == null) return;
				if (!saveToFile(new File(str))) {
					saveAsActivity();
				}
				updateRecentlyUsed(str);
				if (mDoAfterConfirmSave != null) {
					mDoAfterConfirmSave.doProcess();
					mDoAfterConfirmSave = null;
				}
				break;
			case REQUEST_EDIT_CONTENT:
				str = intent.getStringExtra(TextEditor.KEY_STRING);
				if (str == null) return;
				mContentEditText.setText(str);
				updateContent();
				break;
			case REQUEST_MANAGE_BOOKMARKS:
				int[] intarr = intent.getIntArrayExtra(BookmarkManager.KEY_BOOKMARKS);
				if (intarr == null) return;
				Integer[] newBookmarks = new Integer[intarr.length];
				List<Integer> oldBookmarks = mManager.getBookmarks();
				for (int i=0; i<intarr.length; i++) {
					newBookmarks[i] = oldBookmarks.get(intarr[i]);
				}
				mManager.clearBookmarks();
				if (intarr.length == 0) {
					setButtonEnabled(R.id.ButtonBookmarks, false);
				} else {
					setButtonEnabled(R.id.ButtonBookmarks, true);
				}
				for (int i=0; i<intarr.length; i++) {
					mManager.addBookmark(newBookmarks[i]);
				}
				break;
			case REQUEST_HOIST:
				@SuppressWarnings("unchecked")
				ArrayList<ArrayList<InMemoryTreeStateEditInfo<Integer>>> editCommands = (ArrayList<ArrayList<InMemoryTreeStateEditInfo<Integer>>>)intent.getSerializableExtra(HoistedTreeNote.KEY_COMMANDS);
				Trace.d("edit commands: " + editCommands);
				int cmdsSize = editCommands.size();
				if (cmdsSize == 0) return;

				for (int i=0; i<cmdsSize; i++) {
					List<InMemoryTreeStateEditInfo<Integer>> cmd = editCommands.get(i);
					mManager.commitCommand(cmd);
				}

				int redoNum = intent.getIntExtra(HoistedTreeNote.KEY_REDONUM, 0);
				for (int i=0; i<redoNum; i++) {
					mManager.performUndo();
				}

				setUndoRedoButtons();
				break;
			default:
				break;
			}
		}
	}

	private void showConfirmSaveDialog(DoAfterConfirmSave process) {
		mDoAfterConfirmSave = process;
		showDialog(DIALOG_CONFIRM_SAVE);
	}

	@Override
	protected Dialog onCreateDialog(int dialog_id) {
		final Dialog dialog;
		final EditText editText;
		switch (dialog_id) {
		case DIALOG_EDIT:
			editText = new EditText(this);
			editText.setMinWidth(mTreeView.getWidth());
			editText.setMaxWidth(mTreeView.getWidth());
			//~ editText.setSingleLine(true);
			final Integer id_to_edit = mAdapter.getSelectedId();
			editText.setText(mManager.getSubject(id_to_edit));
			dialog = new AlertDialog.Builder(this)
				.setView(editText)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						String str = editText.getText().toString();
						if (mManager.commitCommand(mManager.editCommand_changeSubject(id_to_edit, str))) {
							setUndoRedoButtons();
						}
					}
				})
				.setNegativeButton(R.string.label_CANCEL, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.setCancelable(true)
				.create();
			if (mDontDim) {
				dialog.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND);
			}
			editText.setOnKeyListener(new View.OnKeyListener() {
				@Override
				public boolean onKey(View v, int keyCode, KeyEvent event) {
					if (keyCode == KeyEvent.KEYCODE_ENTER) {
						if (event.getAction() == KeyEvent.ACTION_UP) {
							((AlertDialog)dialog).getButton(DialogInterface.BUTTON_POSITIVE).requestFocus();
						}
						return true;
					}
					return false;
				}
			});
			break;
		case DIALOG_SEARCH:
			editText = new EditText(this);
			editText.setMinWidth(mTreeView.getWidth());
			editText.setMaxWidth(mTreeView.getWidth());
			editText.setSingleLine(true);
			editText.setText(mAdapter.getQueryWord());
			dialog = new AlertDialog.Builder(this)
				.setView(editText)
				.setTitle(R.string.label_search)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						String query = editText.getText().toString();
						if (query.length() == 0) {
							mAdapter.setSearch(null);
						} else {
							mAdapter.setSearch(query);
							performCommand(COMMAND_SEARCH_NEXT);
						}
					}
				})
				.setNegativeButton(R.string.label_CANCEL, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_REPLACE:
			editText = new EditText(this);
			editText.setMinWidth(mTreeView.getWidth());
			editText.setMaxWidth(mTreeView.getWidth());
			editText.setSingleLine(true);
			final String query = mAdapter.getQueryWord();
			dialog = new AlertDialog.Builder(this)
				.setView(editText)
				.setTitle(getString(R.string.label_replace_title, query))
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						String replace = editText.getText().toString();
						if (mManager.commitCommand(mManager.editCommand_replaceAll(query, replace))) {
							setUndoRedoButtons();
						}
						mAdapter.setSearch(null);
					}
				})
				.setNegativeButton(R.string.label_CANCEL, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_CONFIRM_SAVE:
			// show this dialog only by showConfirmSaveDialog()
			dialog = new AlertDialog.Builder(this)
				.setMessage(getString(R.string.message_confirm_save))
				.setNegativeButton(getString(R.string.label_NO), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						if (mDoAfterConfirmSave != null) {
							mDoAfterConfirmSave.doProcess();
						} else {
							Log.e("TreeNote", "error: mDoAfterConfirmSave == null");
						}
					}
				})
				.setPositiveButton(getString(R.string.label_YES), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						performCommand(COMMAND_SAVE);
					}
				})
				.setOnCancelListener(new DialogInterface.OnCancelListener() {
					public void onCancel(DialogInterface dialog) {
						mDoAfterConfirmSave = null;
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_RECENTLY_USED:
			dialog = new AlertDialog.Builder(this)
				.setTitle(getString(R.string.label_recently_used))
				.setItems(mRecentlyUsedFiles.toArray(new String[0]), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						String str = mRecentlyUsedFiles.get(which);
						loadFile(new File(str));
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_UI_SETTING:
			LayoutInflater inflater = LayoutInflater.from(TreeNote.this);
			View layout = inflater.inflate(R.layout.ui_dialog, null);

			final EditText editText_contentWidth = (EditText)layout.findViewById(R.id.ContentWidthEditText);
			editText_contentWidth.setText(String.valueOf((int)((1.0F - mContentSplitView.getSplitterPositionPercent())*100.0F)));
			editText_contentWidth.setFilters(new InputFilter[] {new PercentFilter()} );
			final EditText editText_newTextWidth = (EditText)layout.findViewById(R.id.AddNewWidthEditText);
			editText_newTextWidth.setText(String.valueOf((int)((1.0F - mAddNewSplitView.getSplitterPositionPercent())*100.0F)));
			editText_newTextWidth.setFilters(new InputFilter[] {new PercentFilter()} );
			final CheckBox checkBox_singleLineSubject = (CheckBox)layout.findViewById(R.id.CheckBoxSingleLineSubject);
			checkBox_singleLineSubject.setChecked(mAdapter.getSingleLineMode());

			TextView tv1 = (TextView)layout.findViewById(R.id.ContentWidthLabel);
			tv1.setTextSize(checkBox_singleLineSubject.getTextSize());
			tv1.setTextColor(checkBox_singleLineSubject.getCurrentTextColor());
			TextView tv2 = (TextView)layout.findViewById(R.id.AddNewWidthLabel);
			tv2.setTextSize(checkBox_singleLineSubject.getTextSize());
			tv2.setTextColor(checkBox_singleLineSubject.getCurrentTextColor());

        	dialog = new AlertDialog.Builder(this)
				.setTitle(getString(R.string.menu_interface))
				.setView(layout)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						mContentSplitView.setSplitterPositionPercent(1.0F - Float.valueOf(editText_contentWidth.getText().toString())/100.0F);
						mAddNewSplitView.setSplitterPositionPercent(1.0F - Float.valueOf(editText_newTextWidth.getText().toString())/100.0F);
						mAdapter.setSingleLineMode(checkBox_singleLineSubject.isChecked());
						mAdapter.refresh();
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_CREATE_SHORTCUT:
			dialog = new AlertDialog.Builder(this)
				.setMessage(getString(R.string.message_confirm_create_shortcut, mFileName))
				.setNegativeButton(getString(R.string.label_NO), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.setPositiveButton(getString(R.string.label_YES), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						Intent shortcutIntent = new Intent(Intent.ACTION_VIEW);
						shortcutIntent.setClassName(TreeNote.this, TreeNote.class.getName());
						shortcutIntent.setData(Uri.fromFile(new File(mDirName + "/" + mFileName)));

						Intent intent = new Intent();
						intent.putExtra(Intent.EXTRA_SHORTCUT_INTENT, shortcutIntent);
						intent.putExtra(Intent.EXTRA_SHORTCUT_NAME, mFileName);
						//~ Parcelable iconResource = Intent.ShortcutIconResource.fromContext(this, R.drawable.icon);
						//~ intent.putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, iconResource);
						intent.setAction("com.android.launcher.action.INSTALL_SHORTCUT");
						sendBroadcast(intent);
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_BOOKMARKS:
			final List<Integer> bookmarks = mManager.getBookmarks();
			final int b_size = bookmarks.size();
			String[] items = new String[b_size+1];
			for (int i=0; i<b_size; i++) {
				items[i] = mManager.getSubject(bookmarks.get(i));
			}
			items[b_size] = getString(R.string.label_manage_bookmarks);
			dialog = new AlertDialog.Builder(this)
				.setTitle(getString(R.string.label_bookmarks))
				.setItems(items, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						if (which == b_size) {
							performCommand(COMMAND_MANAGE_BOOKMARKS);
						} else {
							setReturnID(mAdapter.getSelectedId());
							Integer id = bookmarks.get(which);
							mManager.expandToId(id);
							select(id);
						}
					}
				})
				.setCancelable(true)
				.create();
			break;
		case DIALOG_SPLIT_SUBJECT:
			editText = new LockedEditText(TreeNote.this);
			editText.setText(mManager.getSubject(mSplitID));
			dialog = new AlertDialog.Builder(this)
				.setTitle(getString(R.string.label_split_region))
				.setView(editText)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						int start = editText.getSelectionStart();
						int end = editText.getSelectionEnd();
						if (start != end) {
							mSplitPos[0] = Math.min(start, end);
							mSplitPos[1] = Math.max(start, end);
						}
						removeDialog(TreeNote.DIALOG_SPLIT_CONTENT);
						showDialog(TreeNote.DIALOG_SPLIT_CONTENT);
					}
				})
				.setNegativeButton(R.string.label_CANCEL, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				})
				.setCancelable(true)
				.create();
			editText.setOnKeyListener(new View.OnKeyListener() {
				@Override
				public boolean onKey(View v, int keyCode, KeyEvent event) {
					if (keyCode == KeyEvent.KEYCODE_ENTER) {
						if (event.getAction() == KeyEvent.ACTION_UP) {
							((AlertDialog)dialog).getButton(DialogInterface.BUTTON_POSITIVE).performClick();
						}
						return true;
					}
					return false;
				}
			});
			break;
		case DIALOG_SPLIT_CONTENT:
			editText = new LockedEditText(TreeNote.this);
			String str = mManager.getContent(mSplitID);
			if (str.length() == 0) {
				dialog = null;
				mSplitPos[2] = 0;
				mSplitPos[3] = 0;
				splitNode();
			} else {
				editText.setText(str);
				dialog = new AlertDialog.Builder(this)
					.setView(editText)
					.setTitle(getString(R.string.label_split_region))
					.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int whichButton) {
							int start = editText.getSelectionStart();
							int end = editText.getSelectionEnd();
							if (start != end) {
								mSplitPos[2] = Math.min(start, end);
								mSplitPos[3] = Math.max(start, end);
							}
							splitNode();
						}
					})
					.setNegativeButton(R.string.label_CANCEL, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int whichButton) {
						}
					})
					.setCancelable(true)
					.create();
				editText.setOnKeyListener(new View.OnKeyListener() {
					@Override
					public boolean onKey(View v, int keyCode, KeyEvent event) {
						if (keyCode == KeyEvent.KEYCODE_ENTER) {
							if (event.getAction() == KeyEvent.ACTION_UP) {
								((AlertDialog)dialog).getButton(DialogInterface.BUTTON_POSITIVE).performClick();
							}
							return true;
						}
						return false;
					}
				});
			}
			break;
		default:
			dialog = null;
		}

		return dialog;
	}

	private class PercentFilter implements InputFilter {
		@Override
		public CharSequence filter(CharSequence source, int start, int end, Spanned dest, int dstart, int dend) {
			String destStr = dest.toString();
			String newValue = destStr.substring(0, dstart) + source + destStr.substring(dend);

			int val = 0;
			try {
				val = Integer.parseInt(newValue);
			} catch (Exception e) {

				return "";
			}

			if (val < 0 || 100 < val) {
				return "";
			}

			return source;
		}
	}
}
