package jp.deadend.noname.treenote;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import pl.polidea.treeview.AbstractTreeViewAdapter;
import pl.polidea.treeview.TreeNodeInfo;
import android.app.Activity;
import android.text.TextUtils;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.BackgroundColorSpan;
import android.view.Gravity;
import android.view.View;
import android.widget.TextView;

class TreeNoteAdapter extends AbstractTreeViewAdapter<Integer> {
	private TreeNoteStateManager mManager;
	private Integer mSelected;
	private String mQueryWord;
	private int mFontSize;
	private int mLineSpacing;
	private boolean isSingleLineMode;
	private static final int SEARCH_HIGHLIGHT_COLOR = 0xFFFFBF00;

	public TreeNoteAdapter(final Activity treeNote, final TreeNoteStateManager manager, final int numberOfLevels, int fontsize) {
		super(treeNote, manager, numberOfLevels);
		mManager = manager;
		mSelected = null;
		mQueryWord = null;
		mFontSize = fontsize;
		mLineSpacing = 0;
		isSingleLineMode = false;
	}

	void unSelect() {
		mSelected = null;
		refresh();
		((TreeNote)getActivity()).displayContent("");
	}

	void setSearch(String word) {
		mQueryWord = word;
		refresh();
	}

	String getQueryWord() {
		return mQueryWord;
	}

	Integer getSelectedId() {
		return mSelected;
	}

	String getSelectedSubject() {
		return mManager.getSubject(mSelected);
	}

	void setFontSize(int size) {
		mFontSize = size;
	}

	void setLineSpacing(int lsp) {
		mLineSpacing = lsp;
	}

	void setSingleLineMode(boolean s) {
		isSingleLineMode = s;
	}

	boolean getSingleLineMode() {
		return isSingleLineMode;
	}

	void toggleSingleLineMode() {
		isSingleLineMode = !isSingleLineMode;
		refresh();
	}

	static SpannableString spanMatchedWords(CharSequence text, String query) {
		SpannableString ss = new SpannableString(text);
		Matcher m;

		if (query != null) {
			m = Pattern.compile(query).matcher(text);
			while (m.find()) {
				BackgroundColorSpan bgspan = new BackgroundColorSpan(SEARCH_HIGHLIGHT_COLOR);
				ss.setSpan(bgspan, m.start(), m.end(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
			}
		}

		return ss;
	}

	@Override
	public View getNewChildView(final TreeNodeInfo<Integer> treeNodeInfo) {
		final TextView tv = new TextView(getActivity());
		tv.setGravity(Gravity.CENTER_VERTICAL|Gravity.LEFT);

		return updateView(tv, treeNodeInfo);
	}

	@Override
	public View updateView(final View view, final TreeNodeInfo<Integer> treeNodeInfo) {
		final TextView tv = (TextView)view;
		Integer id = treeNodeInfo.getId();
		String subj = mManager.getSubject(treeNodeInfo.getId());
		if (mQueryWord != null) {
			if (subj.contains(mQueryWord)) {
				tv.setText(spanMatchedWords(subj, mQueryWord));
			} else {
				tv.setText(subj);
			}
		} else {
			tv.setText(subj);
		}
		tv.setTextSize(mFontSize);
		tv.setPadding(2, mLineSpacing, 4, mLineSpacing);
		if (isSingleLineMode) {
			tv.setSingleLine();
			tv.setEllipsize(TextUtils.TruncateAt.END);
		} else {
			tv.setSingleLine(false);
			tv.setEllipsize(null);
		}

		if (id.equals(mSelected)) {
			tv.setTextColor(0xFF000000);
			tv.setBackgroundResource(R.drawable.list_selector_selected);
		} else if (mManager.isBookmarked(id)) {
			tv.setTextColor(0xFFFFFFFF);
			tv.setBackgroundColor(0xFFFF0000);
		} else {
			tv.setTextColor(0xFFFFFFFF);
			//~ view.setBackgroundColor(0x00000000);
			tv.setBackgroundResource(0);
		}


		return tv;
	}

	@Override
	public void handleItemClick(final View view, final Object id) {
		TreeNote act = (TreeNote)getActivity();
		act.updateContent();
		act.setFocusToTreeView();

		if (mSelected != null && mSelected.equals(id)) {
	        final TreeNodeInfo<Integer> info = getManager().getNodeInfo((Integer)id);
	        if (info.isWithChildren()) {
	            super.handleItemClick(view, id);
	        }
		} else {
			displayContent((Integer)id);
			mSelected = (Integer) id;
		}
		refresh();
	}

	@Override
	public boolean handleItemLongClick(final View view, final Object id) {
		Activity act = getActivity();
		((TreeNote)act).updateContent();
		displayContent((Integer)id);
		mSelected = (Integer)id;

		((TreeNote)act).performCommand(TreeNote.COMMAND_EDIT_NODE);

		return true;
	}

	@Override
	public void handleItemSelect(final View view, final Object id) {
		((TreeNote)getActivity()).updateContent();
		displayContent((Integer)id);
		mSelected = (Integer)id;
		refresh();
	}

	private void displayContent(Integer id) {
		TreeNote act = (TreeNote)getActivity();
		String cont = mManager.getContent(id);
		if (mQueryWord != null) {
			if (cont.contains(mQueryWord)) {
				act.displayContent(spanMatchedWords(cont, mQueryWord));
			} else {
				act.displayContent(cont);
			}
		} else {
			act.displayContent(cont);
		}
	}

	@Override
	public long getItemId(final int position) {
		return getTreeId(position);
	}
}