package jp.deadend.noname.treenote;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.view.KeyEvent;
import android.view.Window;

public class TreeNotePrefs extends PreferenceActivity {
	private static Map<String, Integer> DEFAULT_SHORTCUTS = null;

	private static void buildDefaultShortcuts(Context ctx) {
		if (DEFAULT_SHORTCUTS != null) return;

		Map<String, Integer> m = new HashMap<String, Integer>();
		m.put(ctx.getString(R.string.pref_key_promote), KeyEvent.KEYCODE_H);
		m.put(ctx.getString(R.string.pref_key_demote), KeyEvent.KEYCODE_L);
		m.put(ctx.getString(R.string.pref_key_moveup), KeyEvent.KEYCODE_K);
		m.put(ctx.getString(R.string.pref_key_movedown), KeyEvent.KEYCODE_J);
		m.put(ctx.getString(R.string.pref_key_mergeprev), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_mergenext), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_split), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_cut), KeyEvent.KEYCODE_X);
		m.put(ctx.getString(R.string.pref_key_copy), KeyEvent.KEYCODE_C);
		m.put(ctx.getString(R.string.pref_key_paste), KeyEvent.KEYCODE_V);
		m.put(ctx.getString(R.string.pref_key_undo), KeyEvent.KEYCODE_Z);
		m.put(ctx.getString(R.string.pref_key_redo), KeyEvent.KEYCODE_Y);
		m.put(ctx.getString(R.string.pref_key_switch), KeyEvent.KEYCODE_T);
		m.put(ctx.getString(R.string.pref_key_save), KeyEvent.KEYCODE_S);
		m.put(ctx.getString(R.string.pref_key_new), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_open), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_editnode), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_editcontent), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_togglecontent), KeyEvent.KEYCODE_P);
		m.put(ctx.getString(R.string.pref_key_togglenew), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_searchprev), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_searchnext), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_addbookmark), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_bookmarks), KeyEvent.KEYCODE_UNKNOWN);
		m.put(ctx.getString(R.string.pref_key_back), KeyEvent.KEYCODE_UNKNOWN);
		DEFAULT_SHORTCUTS = Collections.unmodifiableMap(m);
	}

	@Override
	protected void onCreate(Bundle icicle) {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		super.onCreate(icicle);
		addPreferencesFromResource(R.xml.pref);
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this);

		OnPreferenceChangeListener listener = new OnPreferenceChangeListener() {
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {
				if (newValue != null) {
					if (preference instanceof ListPreference) {
						int listId = ((ListPreference)preference).findIndexOfValue((String)newValue);
						CharSequence[] entries = ((ListPreference)preference).getEntries();
						preference.setSummary(entries[listId]);
					} else if (preference instanceof SetKeyPreference) {
						preference.setSummary(SetKeyPreference.getKeyName((Integer)newValue));
					} else {
						preference.setSummary((CharSequence)newValue);
					}
					return true;
				}
				return false;
			}
		};

		ListPreference fontPr = (ListPreference)findPreference(getString(R.string.pref_key_fontsize));
		fontPr.setSummary(fontPr.getEntry());
		fontPr.setOnPreferenceChangeListener(listener);

		EditTextPreference indentPr = (EditTextPreference)findPreference(getString(R.string.pref_key_indentstr));
		indentPr.setSummary(indentPr.getText());
		indentPr.setOnPreferenceChangeListener(listener);

		EditTextPreference lspPr = (EditTextPreference)findPreference(getString(R.string.pref_key_linespacing));
		lspPr.setSummary(lspPr.getText());
		lspPr.setOnPreferenceChangeListener(listener);

		EditTextPreference splPr = (EditTextPreference)findPreference(getString(R.string.pref_key_splittersize));
		splPr.setSummary(splPr.getText());
		splPr.setOnPreferenceChangeListener(listener);

		ListPreference toolbarPr = (ListPreference)findPreference(getString(R.string.pref_key_toolbarpos));
		toolbarPr.setSummary(toolbarPr.getEntry());
		toolbarPr.setOnPreferenceChangeListener(listener);

		ListPreference toolbarSizePr = (ListPreference)findPreference(getString(R.string.pref_key_toolbarsize));
		toolbarSizePr.setSummary(toolbarSizePr.getEntry());
		toolbarSizePr.setOnPreferenceChangeListener(listener);

		ListPreference encodingPr = (ListPreference)findPreference(getString(R.string.pref_key_encoding));
		encodingPr.setSummary(encodingPr.getEntry());
		encodingPr.setOnPreferenceChangeListener(listener);

		ListPreference newlinePr = (ListPreference)findPreference(getString(R.string.pref_key_newline));
		newlinePr.setSummary(newlinePr.getEntry());
		newlinePr.setOnPreferenceChangeListener(listener);

		ListPreference modkeyPr = (ListPreference)findPreference(getString(R.string.pref_key_modkey));
		modkeyPr.setSummary(modkeyPr.getEntry());
		modkeyPr.setOnPreferenceChangeListener(listener);

		buildDefaultShortcuts(this);
		for (String key : DEFAULT_SHORTCUTS.keySet()) {
			SetKeyPreference skp = (SetKeyPreference)findPreference(key);
			if (!sp.contains(key)) {
				sp.edit().putInt(key, DEFAULT_SHORTCUTS.get(key)).commit();
			}
			skp.setSummary(SetKeyPreference.getKeyName(sp.getInt(key, DEFAULT_SHORTCUTS.get(key))));
			skp.setOnPreferenceChangeListener(listener);
		}
	}

	static String getLastDir(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_lastdir), null);
	}

	static String getLastFile(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_lastfile), null);
	}

	static String getLastExpansionState(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_lastexpansionstate), null);
	}

	static int getLastPosition(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getInt(context.getString(R.string.pref_key_lastposition), 0);
	}

	static float getNewTextSplitterPosition(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getFloat(context.getString(R.string.pref_key_splitternew), -1);
	}

	static float getContentSplitterPosition(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getFloat(context.getString(R.string.pref_key_splittercontent), -1);
	}

	static boolean getIsSingleLineMode(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_issingleline), false);
	}

	static String getFontSize(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_fontsize), context.getString(R.string.pref_value_fontsize_default));
	}

	static String getLineSpacing(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_linespacing), context.getString(R.string.pref_value_linespacing_default));
	}

	static String getSplitterSize(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_splittersize), context.getString(R.string.pref_value_splittersize_default));
	}

	//~ static boolean getToolbar(Context context) {
		//~ return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_toolbar), true);
	//~ }

	static String getToolbar(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_toolbarpos), context.getString(R.string.pref_value_toolbar_default));
	}

	static String getToolbarSize(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_toolbarsize), context.getString(R.string.pref_value_toolbarsize_default));
	}

	static boolean getAutoHideButtons(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_autohidebuttons), false);
	}

	static boolean getSeekbar(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_seekbar), false);
	}

	static boolean getDontDim(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_dontdim), false);
	}

	static boolean getCollapsedOnLoad(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(context.getString(R.string.pref_key_collapsedonload), false);
	}

	static String getIndentStr(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_indentstr), context.getString(R.string.pref_value_indentstr_default));
	}

	static String getEncoding(Context context) {
		return PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_encoding), context.getString(R.string.pref_value_encoding_default));
	}

	static String getNewLineChar(Context context) {
		String defstr = context.getString(R.string.pref_value_newline_default);
		String str = PreferenceManager.getDefaultSharedPreferences(context).getString(context.getString(R.string.pref_key_newline), defstr);

		String ret = null;
		try {
			ret = NewLine.valueOf(str).getVal();
		} catch (IllegalArgumentException e) {
			ret = NewLine.valueOf(defstr).getVal();
			PreferenceManager.getDefaultSharedPreferences(context).edit().remove(context.getString(R.string.pref_key_newline)).commit();
		}

		return ret;
	}

	static String getModKey(Context context) {
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
		String str = null;
		try {
			str = sp.getString(context.getString(R.string.pref_key_modkey), context.getString(R.string.pref_value_modkey_default));
		} catch (ClassCastException e) {
			sp.edit().remove(context.getString(R.string.pref_key_modkey)).commit();
			str = context.getString(R.string.pref_value_modkey_default);
		}
		return str;
	}

	static int getShortcut(Context context, String key) {
		buildDefaultShortcuts(context);
		if (DEFAULT_SHORTCUTS.get(key) == null) return -1;
		return PreferenceManager.getDefaultSharedPreferences(context).getInt(key, DEFAULT_SHORTCUTS.get(key));
	}
}
