package pl.polidea.treeview;

import java.io.Serializable;

public class InMemoryTreeStateEditInfo<T extends Serializable> implements Serializable {
    private static final long serialVersionUID = 1L;

	public enum EditType {
		NONE,
		ADD_CHILD,
		REMOVE_NODE,
		CHANGE_PARENT,
		MOVE_UP,
		MOVE_DOWN,
		RAISE,
		LOWER,
	}

	private EditType type;
	private T id1;
	private T id2;
	private T id3;
	private int index1;
	private int index2;

	public InMemoryTreeStateEditInfo() {
		type = EditType.NONE;
		id1 = null;
		id2 = null;
		id3 = null;
		index1 = -1;
		index2 = -1;
	}
	private InMemoryTreeStateEditInfo(EditType tp, T i1, T i2, T i3, int idx1, int idx2) {
		type = tp;
		id1 = i1;
		id2 = i2;
		id3 = i3;
		index1 = idx1;
		index2 = idx2;
	}

	public void setAddChild(T id, T childId, int index) {
		type=EditType.ADD_CHILD;		id1=id;	id2=childId;	id3=null;			index1=index;		index2=-1;
	}
	public void setRemoveNode(T id, T parentId, int index) {
		type=EditType.REMOVE_NODE;		id1=id;	id2=parentId;	id3=null;			index1=index;		index2=-1;
	}
	public void setChangeParent(T id, T oldParentId, T newParentId, int oldIndex, int newIndex) {
		type=EditType.CHANGE_PARENT;	id1=id;	id2=oldParentId;id3=newParentId;	index1=oldIndex;	index2=newIndex;
	}
	public void setMoveUp(T id) {
		type=EditType.MOVE_UP;			id1=id;	id2=null;		id3=null;			index1=-1;			index2=-1;
	}
	public void setMoveDown(T id) {
		type=EditType.MOVE_DOWN;		id1=id;	id2=null;		id3=null;			index1=-1;			index2=-1;
	}
	public void setRaise(T id) {
		type=EditType.RAISE;			id1=id;	id2=null;		id3=null;			index1=-1;			index2=-1;
	}
	public void setLower(T id) {
		type=EditType.LOWER;			id1=id;	id2=null;		id3=null;			index1=-1;			index2=-1;
	}

	public InMemoryTreeStateEditInfo<T> createClone() {
		return new InMemoryTreeStateEditInfo<T>(type, id1, id2, id3, index1, index2);
	}

	public void convertRootInHoisted(T newId) {
		if (type == EditType.ADD_CHILD) {
			if (id1 == null) {id1 = newId;}
		} else if (type == EditType.REMOVE_NODE) {
			if (id2 == null) {id2 = newId;}
		} else if (type == EditType.CHANGE_PARENT) {
			if (id2 == null) {id2 = newId;}
			if (id3 == null) {id3 = newId;}
		}
	}

	public String toString() {
		switch (type) {
		case NONE:
			return "NONE";
		case ADD_CHILD:
			return "ADD_CHILD:"+id1+","+id2+","+index1;
		case REMOVE_NODE:
			return "REMOVE_NODE:"+id1+","+id2+","+index1;
		case CHANGE_PARENT:
			return "CHANGE_PARENT:"+id1+","+id2+","+id3+","+index1+","+index2;
		case MOVE_UP:
			return "MOVE_UP:"+id1;
		case MOVE_DOWN:
			return "MOVE_DOWN:"+id1;
		case RAISE:
			return "RAISE:"+id1;
		case LOWER:
			return "LOWER:"+id1;
		}

		return "UNKNOWN";
	}

	public EditType getType() {
		return type;
	}
	public T getId1() {
		return id1;
	}
	public T getId2() {
		return id2;
	}
	public T getId3() {
		return id3;
	}
	public int getIndex1() {
		return index1;
	}
	public int getIndex2() {
		return index2;
	}

}
