package pl.polidea.treeview;

import java.io.Serializable;

public class TreeNoteBuilder<T extends Serializable> {
    private final InMemoryTreeStateManager<T> manager;

    private T lastAddedId = null;
    private int lastLevel = -1;

    public TreeNoteBuilder(final InMemoryTreeStateManager<T> manager) {
        this.manager = manager;
    }

    public synchronized void sequentiallyAddNextNode(final T id, final int level) {
        if (lastAddedId == null) {
            addNodeToParent(null, id, level);
        } else {
            if (level <= lastLevel) {
                final T parent = findParentAtLevel(lastAddedId, level - 1);
                addNodeToParent(parent, id, level);
            } else {
                addNodeToParent(lastAddedId, id, level);
            }
        }
    }

    private T findParentAtLevel(final T node, final int levelToFind) {
        T parent = manager.getParent(node);
        while (parent != null) {
            if (manager.getLevel(parent) == levelToFind) {
                break;
            }
            parent = manager.getParent(parent);
        }
        return parent;
    }

    private void addNodeToParent(final T parent, final T id, final int level) {
        manager.addAfterChild(parent, id, null);
		while (manager.getLevel(id) < level) {
			manager.lowerLevelSubtree(id);
		}

        lastAddedId = id;
        lastLevel = level;
    }
}
